@@ -7,27 +7,28 @@
 ;; Copyright (C) 1996-2009, Drew Adams, all rights reserved.
 ;; Created: Mon Feb 27 09:19:43 2006
 ;; Version: 22.0
-;; Last-Updated: Tue Jan 13 21:02:04 2009 (-0800)
+;; Last-Updated: Wed Feb 17 09:17:51 2010 (-0800)
 ;;           By: dradams
-;;     Update #: 493
+;;     Update #: 512
 ;; URL: http://www.emacswiki.org/cgi-bin/wiki/icicles-face.el
 ;; Keywords: internal, extensions, help, abbrev, local, minibuffer,
 ;;           keys, apropos, completion, matching, regexp, command
-;; Compatibility: GNU Emacs 20.x, GNU Emacs 21.x, GNU Emacs 22.x
+;; Compatibility: GNU Emacs: 20.x, 21.x, 22.x, 23.x
 ;;
 ;; Features that might be required by this library:
 ;;
-;;   `cl', `color-theme', `cus-face', `easymenu', `ffap', `ffap-',
-;;   `hexrgb', `icicles-opt', `kmacro', `thingatpt', `thingatpt+',
-;;   `wid-edit', `widget'.
+;;   `cl', `color-theme', `cus-face', `easymenu', `el-swank-fuzzy',
+;;   `ffap', `ffap-', `fuzzy-match', `hexrgb', `icicles-opt',
+;;   `kmacro', `levenshtein', `reporter', `sendmail', `thingatpt',
+;;   `thingatpt+', `wid-edit', `widget'.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;;; Commentary:
 ;;
 ;;  This is a helper library for library `icicles.el'.  It defines
-;;  customization groups and faces.  See `icicles.el' for
-;;  documentation.
+;;  customization groups and faces.  For Icicles documentation, see
+;;  `icicles-doc1.el' and `icicles-doc2.el'.
 ;;
 ;;  Groups defined here:
 ;;
@@ -48,7 +49,7 @@
 ;;    `icicle-historical-candidate', `icicle-input-completion-fail',
 ;;    `icicle-input-completion-fail-lax',
 ;;    `icicle-match-highlight-Completions',
-;;    `icicle-match-highlight-minibuffer',
+;;    `icicle-match-highlight-minibuffer', `icicle-mode-line-help',
 ;;    `icicle-multi-command-completion',
 ;;    `icicle-mustmatch-completion', `icicle-proxy-candidate',
 ;;    `icicle-saved-candidate', `icicle-search-context-level-1',
@@ -63,6 +64,11 @@
 ;;    `icicle-search-main-regexp-others', `icicle-special-candidate',
 ;;    `icicle-whitespace-highlight', `minibuffer-prompt'.
 ;;
+;;  Functions defined here:
+;;
+;;    `icicle-increment-color-hue',
+;;    `icicle-increment-color-saturation'.
+;;
 ;;  For descriptions of changes to this file, see `icicles-chg.el'.
  
 ;;(@> "Index")
@@ -98,9 +104,6 @@
 ;;
 ;;; Code:
 
-(require 'icicles-opt) ;; icicle-increment-color-hue,
-                       ;; icicle-increment-color-saturation
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
 ;;(@* "Groups, organized alphabetically")
@@ -373,6 +376,12 @@ This means that they belong to list `icicle-extra-candidates'."
   "*Face used to highlight root that was completed, in minibuffer."
   :group 'Icicles-Minibuffer-Display :group 'faces)
 
+(defface icicle-mode-line-help
+  '((((background dark)) (:foreground "#AC4AAC4A0000")) ; a dark yellow
+    (t (:foreground "Blue")))
+  "*Face used to highlight help shown in the mode-line."
+  :group 'Icicles-Completions-Display :group 'Icicles-Miscellaneous :group 'faces)
+
 (defface icicle-multi-command-completion
     '((((background dark)) ; a dark cyan on a dark magenta
        (:foreground "#0000D53CD53C" :background "#8B3500007533"))
@@ -413,6 +422,49 @@ Not used for versions of Emacs before version 21."
 This highlighting is done during Icicles searching."
   :group 'Icicles-Searching :group 'faces)
 
+;; This is essentially a version of `doremi-increment-color-component' for hue only.
+;; Must be before `icicle-search-context-level-1'.
+(defun icicle-increment-color-hue (color increment)
+  "Increase hue component of COLOR by INCREMENT."
+  (unless (featurep 'hexrgb) (error "`icicle-increment-color-hue' requires library `hexrgb.el'"))
+  (unless (string-match "#" color)      ; Convert color name to #hhh...
+    (setq color  (hexrgb-color-values-to-hex (x-color-values color))))
+  ;; Convert RGB to HSV
+  (let* ((rgb         (x-color-values color))
+         (red         (/ (float (nth 0 rgb)) 65535.0)) ; Convert from 0-65535 to 0.0-1.0
+         (green       (/ (float (nth 1 rgb)) 65535.0))
+         (blue        (/ (float (nth 2 rgb)) 65535.0))
+         (hsv         (hexrgb-rgb-to-hsv red green blue))
+         (hue         (nth 0 hsv))
+         (saturation  (nth 1 hsv))
+         (value       (nth 2 hsv)))
+    (setq hue  (+ hue (/ increment 100.0)))
+    (when (> hue 1.0) (setq hue  (1- hue)))
+    (hexrgb-color-values-to-hex (mapcar (lambda (x) (floor (* x 65535.0)))
+                                        (hexrgb-hsv-to-rgb hue saturation value)))))
+
+;; This is essentially a version of `doremi-increment-color-component' for saturation only.
+;; Must be before `icicle-search-context-level-1'.
+(defun icicle-increment-color-saturation (color increment)
+  "Increase saturation component of COLOR by INCREMENT."
+  (unless (featurep 'hexrgb)
+    (error "`icicle-increment-color-saturation' requires library `hexrgb.el'"))
+  (unless (string-match "#" color)      ; Convert color name to #hhh...
+    (setq color  (hexrgb-color-values-to-hex (x-color-values color))))
+  ;; Convert RGB to HSV
+  (let* ((rgb         (x-color-values color))
+         (red         (/ (float (nth 0 rgb)) 65535.0)) ; Convert from 0-65535 to 0.0-1.0
+         (green       (/ (float (nth 1 rgb)) 65535.0))
+         (blue        (/ (float (nth 2 rgb)) 65535.0))
+         (hsv         (hexrgb-rgb-to-hsv red green blue))
+         (hue         (nth 0 hsv))
+         (saturation  (nth 1 hsv))
+         (value       (nth 2 hsv)))
+    (setq saturation  (+ saturation (/ increment 100.0)))
+    (when (> saturation 1.0) (setq saturation  (1- saturation)))
+    (hexrgb-color-values-to-hex (mapcar (lambda (x) (floor (* x 65535.0)))
+                                        (hexrgb-hsv-to-rgb hue saturation value)))))
+
 (defface icicle-search-context-level-1
     (let ((context-bg  (face-background 'icicle-search-main-regexp-current)))
       `((((background dark))
